@@ -1,7 +1,7 @@
 // drumkv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -38,6 +38,8 @@
 #include "drumkv1_controls.h"
 #include "drumkv1_programs.h"
 
+#include "drumkv1_sched.h"
+
 
 #ifdef CONFIG_DEBUG_0
 #include <stdio.h>
@@ -623,6 +625,11 @@ public:
 
 	drumkv1_element element;
 
+	void reset();
+
+	void midiInEnabled(bool on);
+	uint32_t midiInCount();
+
 	drumkv1_sample  gen1_sample;
 	drumkv1_wave_lf lfo1_wave;
 
@@ -749,6 +756,40 @@ struct drumkv1_voice : public drumkv1_list<drumkv1_voice>
 };
 
 
+// MIDI input asynchronous status notification
+
+class drumkv1_midi_in : public drumkv1_sched
+{
+public:
+
+	drumkv1_midi_in (drumkv1 *pDrumk)
+		: drumkv1_sched(pDrumk, MidiIn),
+			m_enabled(false), m_count(0) {}
+
+	void schedule_event()
+		{ if (m_enabled && ++m_count < 2) schedule(-1); }
+	void schedule_note(int key, int vel)
+		{ if (m_enabled) schedule((vel << 7) | key); }
+
+	void process(int) {}
+
+	void enabled(bool on)
+		{ m_enabled = on; m_count = 0; }
+
+	uint32_t count()
+	{
+		const uint32_t ret = m_count;
+		m_count = 0;
+		return ret;
+	}
+
+private:
+
+	bool     m_enabled;
+	uint32_t m_count;
+};
+
+
 // synth engine implementation
 
 class drumkv1_impl
@@ -806,6 +847,11 @@ public:
 	void resetParamValues(bool bSwap);
 	void reset();
 
+	void midiInEnabled(bool on);
+	uint32_t midiInCount();
+
+	void directNoteOn(int note, int vel);
+
 protected:
 
 	void allSoundOff();
@@ -848,6 +894,7 @@ private:
 	drumkv1_config   m_config;
 	drumkv1_controls m_controls;
 	drumkv1_programs m_programs;
+	drumkv1_midi_in  m_midi_in;
 
 	uint16_t m_nchannels;
 	float    m_srate;
@@ -894,14 +941,18 @@ private:
 
 	drumkv1_reverb m_reverb;
 	drumkv1_phasor m_phasor;
+
+	volatile int m_direct_chan;
+	volatile int m_direct_note;
+	volatile int m_direct_vel;
 };
 
 
 // synth engine constructor
 
 drumkv1_impl::drumkv1_impl (
-	drumkv1 *pDrumk, uint16_t nchannels, float srate )
-	: m_pDrumk(pDrumk), m_controls(pDrumk), m_programs(pDrumk), m_bpm(180.0f)
+	drumkv1 *pDrumk, uint16_t nchannels, float srate ) : m_pDrumk(pDrumk),
+		m_controls(pDrumk), m_programs(pDrumk), m_midi_in(pDrumk), m_bpm(180.0f)
 {
 	// allocate voice pool.
 	m_voices = new drumkv1_voice * [MAX_VOICES];
@@ -1475,6 +1526,7 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 					m_group[pv->group] = pv;
 				}
 			}
+			m_midi_in.schedule_note(key, value);
 		}
 		// note off
 		else if (status == 0x80 || (status == 0x90 && value == 0)) {
@@ -1489,6 +1541,7 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 					}
 				}
 			}
+			m_midi_in.schedule_note(key, 0);
 		}
 		// key pressure/poly.aftertouch
 		else if (status == 0xa0) {
@@ -1545,6 +1598,9 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 
 	// process pending controllers...
 	m_controls.process_dequeue();
+
+	// asynchronous event notification...
+	m_midi_in.schedule_event();
 }
 
 
@@ -1597,6 +1653,22 @@ void drumkv1_impl::allNotesOff (void)
 		elem->aux1.reset();
 		elem = elem->next();
 	}
+
+	m_direct_chan = m_direct_note = m_direct_vel = -1;
+}
+
+
+// direct note-on triggered on next cycle...
+void drumkv1_impl::directNoteOn ( int note, int vel )
+{
+	if (vel > 0) {
+		const int ch1 = int(*m_def.channel);
+		m_direct_chan = (ch1 > 0 ? ch1 - 1 : 0) & 0x0f;
+		m_direct_note = note;
+		m_direct_vel  = vel;
+	} else {
+		m_direct_vel  = 0;
+	}
 }
 
 
@@ -1630,6 +1702,22 @@ void drumkv1_impl::resetParamValues ( bool bSwap )
 }
 
 
+// controllers accessor
+
+drumkv1_controls *drumkv1_impl::controls (void)
+{
+	return &m_controls;
+}
+
+
+// programs accessor
+
+drumkv1_programs *drumkv1_impl::programs (void)
+{
+	return &m_programs;
+}
+
+
 // all reset clear
 
 void drumkv1_impl::reset (void)
@@ -1670,22 +1758,19 @@ void drumkv1_impl::reset (void)
 }
 
 
-// controllers accessor
+// MIDI input asynchronous status notification accessors
 
-drumkv1_controls *drumkv1_impl::controls (void)
+void drumkv1_impl::midiInEnabled ( bool on )
 {
-	return &m_controls;
+	m_midi_in.enabled(on);
 }
 
-
-// programs accessor
-
-drumkv1_programs *drumkv1_impl::programs (void)
+uint32_t drumkv1_impl::midiInCount (void)
 {
-	return &m_programs;
+	return m_midi_in.count();
 }
 
-
+ 
 // synthesize
 
 void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
@@ -1703,6 +1788,20 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		::memset(outs[k], 0, nframes * sizeof(float));
 	}
 
+	// process direct note on/off...
+	if (m_direct_chan >= 0 && m_direct_note >= 0 && m_direct_vel >= 0) {
+		struct note_data { uint8_t status, note, vel; } data;
+		data.status = (m_direct_vel > 0 ? 0x90 : 0x80) | m_direct_chan;
+		data.note = m_direct_note;
+		data.vel = m_direct_vel;
+		process_midi((uint8_t *) &data, sizeof(data));
+		if (m_direct_vel == 0) {
+			m_direct_chan = -1;
+			m_direct_note = -1;
+		}
+		m_direct_vel = -1;
+	}
+
 	drumkv1_elem *elem = m_elem_list.next();
 	while (elem) {
 	#if 0
@@ -1942,7 +2041,6 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 }
 
 
-
 //-------------------------------------------------------------------------
 // drumkv1 - decl.
 //
@@ -2309,4 +2407,27 @@ void drumkv1_element::resetParamValues ( bool bSwap )
 }
 
 
+// MIDI input asynchronous status notification accessors
+
+void drumkv1::midiInEnabled ( bool on )
+{
+	m_pImpl->midiInEnabled(on);
+}
+
+
+uint32_t drumkv1::midiInCount (void)
+{
+	return m_pImpl->midiInCount();
+}
+
+
+// MIDI direct note on/off triggering
+
+void drumkv1::directNoteOn ( int note, int vel )
+{
+	m_pImpl->directNoteOn(note, vel);
+}
+
+
 // end of drumkv1.cpp
+
