@@ -1,7 +1,7 @@
 // drumkv1_ui.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -26,7 +26,8 @@
 // drumkv1_ui - decl.
 //
 
-drumkv1_ui::drumkv1_ui ( drumkv1 *pDrumk ) : m_pDrumk(pDrumk)
+drumkv1_ui::drumkv1_ui ( drumkv1 *pDrumk, bool bPlugin )
+	: m_pDrumk(pDrumk), m_bPlugin(bPlugin)
 {
 }
 
@@ -70,6 +71,12 @@ void drumkv1_ui::clearElements (void)
 }
 
 
+bool drumkv1_ui::isPlugin (void) const
+{
+	return m_bPlugin;
+}
+
+
 void drumkv1_ui::setSampleFile ( const char *pszSampleFile )
 {
 	m_pDrumk->setSampleFile(pszSampleFile);
@@ -139,4 +146,22 @@ void drumkv1_ui::updatePreset ( bool bDirty )
 }
 
 
+void drumkv1_ui::midiInEnabled ( bool bEnabled )
+{
+	m_pDrumk->midiInEnabled(bEnabled);
+}
+
+
+uint32_t drumkv1_ui::midiInCount (void)
+{
+	return m_pDrumk->midiInCount();
+}
+
+
+void drumkv1_ui::directNoteOn ( int note, int vel )
+{
+	m_pDrumk->directNoteOn(note, vel);
+}
+
+
 // end of drumkv1_ui.cpp
