@@ -1,7 +1,7 @@
 // drumkv1_ui.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -33,7 +33,11 @@ class drumkv1_ui
 {
 public:
 
-	drumkv1_ui(drumkv1 *pDrumk);
+	drumkv1_ui(drumkv1 *pDrumk, bool bPlugin);
+
+	drumkv1 *instance() const;
+
+	bool isPlugin() const;
 
 	drumkv1_element *addElement(int key);
 	drumkv1_element *element(int key) const;
@@ -52,8 +56,6 @@ public:
 	void setReverse(bool bReverse);
 	bool isReverse() const;
 
-	drumkv1 *instance() const;
-
 	void setParamValue(drumkv1::ParamIndex index, float fValue);
 	float paramValue(drumkv1::ParamIndex index) const;
 
@@ -65,9 +67,16 @@ public:
 
 	void updatePreset(bool bDirty);
 
+	void midiInEnabled(bool bEnabled);
+	uint32_t midiInCount();
+
+	void directNoteOn(int note, int vel);
+
 private:
 
 	drumkv1 *m_pDrumk;
+
+	bool m_bPlugin;
 };
 
 
