@@ -1,7 +1,7 @@
 // drumkv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,8 +28,15 @@
 #include "drumkv1widget_config.h"
 #include "drumkv1widget_control.h"
 
+#include "drumkv1_controls.h"
+#include "drumkv1_programs.h"
+
 #include <QMessageBox>
 #include <QDir>
+#include <QTimer>
+
+#include <QShowEvent>
+#include <QHideEvent>
 
 
 //-------------------------------------------------------------------------
@@ -119,26 +126,30 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		channels << QString::number(iChannel + 1);
 
 	m_ui.Def1ChannelKnob->insertItems(0, channels);
-
+#if 0
 	// Noteoff modes.
 	QStringList modes;
 	modes << tr("Disabled");
 	modes << tr("Enabled");
 
 	m_ui.Def1NoteoffKnob->insertItems(0, modes);
-
+#else
+	m_ui.Def1NoteoffKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
+#endif
 	// Dynamic states.
 	QStringList states;
 	states << tr("Off");
 	states << tr("On");
-
+#if 0
 	m_ui.Gen1ReverseKnob->insertItems(0, states);
 
 	m_ui.Lfo1SyncKnob->insertItems(0, states);
 
 	m_ui.Dyn1CompressKnob->insertItems(0, states);
 	m_ui.Dyn1LimiterKnob->insertItems(0, states);
-
+#else
+	m_ui.Lfo1SyncKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
+#endif
 	// Special values
 	const QString& sOff = states.first();
 	m_ui.Cho1WetKnob->setSpecialValueText(sOff);
@@ -161,7 +172,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Gen1GroupKnob->setMinimum(0.0f);
 	m_ui.Gen1GroupKnob->setMaximum(128.0f);
 	m_ui.Gen1GroupKnob->setDecimals(0);
-	m_ui.Gen1GroupKnob->setSingleStep(10.0f);
+//	m_ui.Gen1GroupKnob->setSingleStep(10.0f);
 
 	// GEN octave limits.
 	m_ui.Gen1CoarseKnob->setMinimum(-4.0f);
@@ -179,7 +190,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Lfo1BpmKnob->setScale(1.0f);
 	m_ui.Lfo1BpmKnob->setMinimum(0.0f);
 	m_ui.Lfo1BpmKnob->setMaximum(360.0f);
-	m_ui.Lfo1BpmKnob->setSingleStep(1.0f);
+//	m_ui.Lfo1BpmKnob->setSingleStep(1.0f);
 	m_ui.Lfo1SweepKnob->setMinimum(-1.0f);
 	m_ui.Lfo1SweepKnob->setMaximum(+1.0f);
 	m_ui.Lfo1CutoffKnob->setMinimum(-1.0f);
@@ -205,7 +216,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Del1BpmKnob->setScale(1.0f);
 	m_ui.Del1BpmKnob->setMinimum(0.0f);
 	m_ui.Del1BpmKnob->setMaximum(360.0f);
-	m_ui.Del1BpmKnob->setSingleStep(1.0f);
+//	m_ui.Del1BpmKnob->setSingleStep(1.0f);
 
 	// GEN1
 	setParamKnob(drumkv1::GEN1_REVERSE, m_ui.Gen1ReverseKnob);
@@ -490,13 +501,15 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	if (pConfig) {
 		drumkv1widget_dial::setDialMode(
 			drumkv1widget_dial::DialMode(pConfig->iKnobDialMode));
+		drumkv1widget_edit::setEditMode(
+			drumkv1widget_edit::EditMode(pConfig->iKnobEditMode));
 	}
 
 	// Epilog.
 	// QWidget::adjustSize();
 
 	m_ui.StatusBar->showMessage(tr("Ready"), 5000);
-	m_ui.StatusBar->setModified(false);
+	m_ui.StatusBar->modified(false);
 	m_ui.Preset->setDirtyPreset(false);
 }
 
@@ -509,13 +522,11 @@ drumkv1widget::~drumkv1widget (void)
 }
 
 
-// Create/initialize the scheduler/work notifier.
-void drumkv1widget::initSchedNotifier (void)
+// Open/close the scheduler/work notifier.
+void drumkv1widget::openSchedNotifier (void)
 {
-	if (m_sched_notifier) {
-		delete m_sched_notifier;
-		m_sched_notifier = NULL;
-	}
+	if (m_sched_notifier)
+		return;
 
 	drumkv1_ui *pDrumkUi = ui_instance();
 	if (pDrumkUi == NULL)
@@ -526,29 +537,61 @@ void drumkv1widget::initSchedNotifier (void)
 	QObject::connect(m_sched_notifier,
 		SIGNAL(notify(int, int)),
 		SLOT(updateSchedNotify(int, int)));
+
+	pDrumkUi->midiInEnabled(true);
+}
+
+
+void drumkv1widget::closeSchedNotifier (void)
+{
+	if (m_sched_notifier) {
+		delete m_sched_notifier;
+		m_sched_notifier = NULL;
+	}
+
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi)
+		pDrumkUi->midiInEnabled(false);
+}
+
+
+// Show/hide widget handlers.
+void drumkv1widget::showEvent ( QShowEvent *pShowEvent )
+{
+	QWidget::showEvent(pShowEvent);
+
+	openSchedNotifier();
+}
+
+
+void drumkv1widget::hideEvent ( QHideEvent *pHideEvent )
+{
+	closeSchedNotifier();
+
+	QWidget::hideEvent(pHideEvent);
 }
 
 
 // Param kbob (widget) map accesors.
-void drumkv1widget::setParamKnob ( drumkv1::ParamIndex index, drumkv1widget_knob *pKnob )
+void drumkv1widget::setParamKnob ( drumkv1::ParamIndex index, drumkv1widget_param *pParam )
 {
-	pKnob->setDefaultValue(drumkv1_param::paramDefaultValue(index));
+	pParam->setDefaultValue(drumkv1_param::paramDefaultValue(index));
 
-	m_paramKnobs.insert(index, pKnob);
-	m_knobParams.insert(pKnob, index);
+	m_paramKnobs.insert(index, pParam);
+	m_knobParams.insert(pParam, index);
 
-	QObject::connect(pKnob,
+	QObject::connect(pParam,
 		SIGNAL(valueChanged(float)),
 		SLOT(paramChanged(float)));
 
-	pKnob->setContextMenuPolicy(Qt::CustomContextMenu);
+	pParam->setContextMenuPolicy(Qt::CustomContextMenu);
 
-	QObject::connect(pKnob,
+	QObject::connect(pParam,
 		SIGNAL(customContextMenuRequested(const QPoint&)),
 		SLOT(paramContextMenu(const QPoint&)));
 }
 
-drumkv1widget_knob *drumkv1widget::paramKnob ( drumkv1::ParamIndex index ) const
+drumkv1widget_param *drumkv1widget::paramKnob ( drumkv1::ParamIndex index ) const
 {
 	return m_paramKnobs.value(index, NULL);
 }
@@ -560,9 +603,9 @@ void drumkv1widget::setParamValue (
 {
 	++m_iUpdate;
 
-	drumkv1widget_knob *pKnob = paramKnob(index);
-	if (pKnob)
-		pKnob->setValue(fValue, bDefault);
+	drumkv1widget_param *pParam = paramKnob(index);
+	if (pParam)
+		pParam->setValue(fValue, bDefault);
 
 	updateParamEx(index, fValue);
 
@@ -573,9 +616,9 @@ float drumkv1widget::paramValue ( drumkv1::ParamIndex index ) const
 {
 	float fValue = 0.0f;
 
-	drumkv1widget_knob *pKnob = paramKnob(index);
-	if (pKnob) {
-		fValue = pKnob->value();
+	drumkv1widget_param *pParam = paramKnob(index);
+	if (pParam) {
+		fValue = pParam->value();
 	} else {
 		drumkv1_ui *pDrumkUi = ui_instance();
 		if (pDrumkUi)
@@ -592,9 +635,9 @@ void drumkv1widget::paramChanged ( float fValue )
 	if (m_iUpdate > 0)
 		return;
 
-	drumkv1widget_knob *pKnob = qobject_cast<drumkv1widget_knob *> (sender());
-	if (pKnob) {
-		const drumkv1::ParamIndex index = m_knobParams.value(pKnob);
+	drumkv1widget_param *pParam = qobject_cast<drumkv1widget_param *> (sender());
+	if (pParam) {
+		const drumkv1::ParamIndex index = m_knobParams.value(pParam);
 		// Save current element param value...
 		drumkv1_ui *pDrumkUi = ui_instance();
 		if (pDrumkUi) {
@@ -607,8 +650,8 @@ void drumkv1widget::paramChanged ( float fValue )
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
 		m_ui.StatusBar->showMessage(QString("%1: %2")
-			.arg(pKnob->toolTip())
-			.arg(pKnob->valueText()), 5000);
+			.arg(pParam->toolTip())
+			.arg(pParam->valueText()), 5000);
 		updateDirtyPreset(true);
 	}
 }
@@ -647,14 +690,14 @@ void drumkv1widget::updateSchedParam ( drumkv1::ParamIndex index, float fValue )
 {
 	++m_iUpdate;
 
-	drumkv1widget_knob *pKnob = paramKnob(index);
-	if (pKnob) {
-		pKnob->setValue(fValue, false);
+	drumkv1widget_param *pParam = paramKnob(index);
+	if (pParam) {
+		pParam->setValue(fValue, false);
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
 		m_ui.StatusBar->showMessage(QString("%1: %2")
-			.arg(pKnob->toolTip())
-			.arg(pKnob->valueText()), 5000);
+			.arg(pParam->toolTip())
+			.arg(pParam->valueText()), 5000);
 		updateDirtyPreset(true);
 	}
 
@@ -678,9 +721,9 @@ void drumkv1widget::resetParams (void)
 		if (index == drumkv1::GEN1_SAMPLE)
 			continue;
 		float fValue = drumkv1_param::paramDefaultValue(index);
-		drumkv1widget_knob *pKnob = paramKnob(index);
-		if (pKnob)
-			fValue = pKnob->defaultValue();
+		drumkv1widget_param *pParam = paramKnob(index);
+		if (pParam && pParam->isDefaultValue())
+			fValue = pParam->defaultValue();
 		setParamValue(index, fValue);
 		updateParam(index, fValue);
 	//	updateParamEx(index, fValue);
@@ -713,10 +756,10 @@ void drumkv1widget::swapParams ( bool bOn )
 				const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 				if (index == drumkv1::GEN1_SAMPLE)
 					continue;
-				drumkv1widget_knob *pKnob = paramKnob(index);
-				if (pKnob) {
-					pKnob->setDefaultValue(element->paramValue(index, 0));
-					element->setParamValue(index, pKnob->value());
+				drumkv1widget_param *pParam = paramKnob(index);
+				if (pParam) {
+					pParam->setDefaultValue(element->paramValue(index, 0));
+					element->setParamValue(index, pParam->value());
 				}
 			}
 		}
@@ -737,9 +780,9 @@ void drumkv1widget::swapParams ( bool bOn )
 		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		if (index == drumkv1::GEN1_SAMPLE)
 			continue;
-		drumkv1widget_knob *pKnob = paramKnob(index);
-		if (pKnob) {
-			const float fOldValue = pKnob->value();
+		drumkv1widget_param *pParam = paramKnob(index);
+		if (pParam) {
+			const float fOldValue = pParam->value();
 			const float fNewValue = m_params_ab[i];
 			setParamValue(index, fNewValue);
 			updateParam(index, fNewValue);
@@ -809,9 +852,9 @@ void drumkv1widget::resetParamKnobs ( uint32_t nparams )
 		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		if (index == drumkv1::GEN1_SAMPLE)
 			continue;
-		drumkv1widget_knob *pKnob = paramKnob(index);
-		if (pKnob)
-			pKnob->resetDefaultValue();
+		drumkv1widget_param *pParam = paramKnob(index);
+		if (pParam)
+			pParam->resetDefaultValue();
 	}
 }
 
@@ -921,6 +964,17 @@ void drumkv1widget::openSample (void)
 }
 
 
+// Sample playback (direct note-on/off).
+void drumkv1widget::playSample (void)
+{
+#ifdef CONFIG_DEBUG
+	qDebug("drumkv1widget::playSample()");
+#endif
+
+	m_ui.Elements->directNoteOn(currentNote());
+}
+
+
 // Sample file reset.
 void drumkv1widget::clearSampleFile (void)
 {
@@ -1068,13 +1122,17 @@ QString drumkv1widget::noteName ( int note )
 
 	// Pre-load drum-names hash table...
 	if (s_names.isEmpty()) {
-		for (int i = 12; s_notes[i].name; ++i) {
-			s_names.insert(s_notes[i].note,
-				QObject::tr(s_notes[i].name, "noteName"));
+		drumkv1_config *pConfig = drumkv1_config::getInstance();
+		if (pConfig && pConfig->bUseGMDrumNames) {
+			for (int i = 12; s_notes[i].name; ++i) {
+				s_names.insert(s_notes[i].note,
+					QObject::tr(s_notes[i].name, "noteName"));
+			}
 		}
 	}
+
 	// Check whether the drum note exists...
-	QHash<int, QString>::ConstIterator iter = s_names.constFind(note);
+	const QHash<int, QString>::ConstIterator& iter = s_names.constFind(note);
 	if (iter != s_names.constEnd())
 		return iter.value();
 
@@ -1199,10 +1257,10 @@ void drumkv1widget::updateElement (void)
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 			const float fValue = element->paramValue(index);
-			drumkv1widget_knob *pKnob = paramKnob(index);
-			if (pKnob) {
-				pKnob->setDefaultValue(element->paramValue(index, 0));
-				pKnob->setValue(fValue);
+			drumkv1widget_param *pParam = paramKnob(index);
+			if (pParam) {
+				pParam->setDefaultValue(element->paramValue(index, 0));
+				pParam->setValue(fValue);
 			}
 			updateParam(index, fValue);
 			m_params_ab[i] = fValue;
@@ -1288,6 +1346,10 @@ void drumkv1widget::contextMenuRequest ( const QPoint& pos )
 		QIcon(":/images/fileOpen.png"),
 		tr("Open Sample..."), this, SLOT(openSample()));
 	pAction->setEnabled(pDrumkUi != NULL);
+	pAction = menu.addAction(
+		QIcon(":/images/playSample.png"),
+		tr("Play"), this, SLOT(playSample()));
+	pAction->setEnabled(element != NULL);
 	menu.addSeparator();
 	pAction = menu.addAction(
 		tr("Reset"), this, SLOT(resetElement()));
@@ -1326,10 +1388,19 @@ void drumkv1widget::updateSchedNotify ( int stype, int sid )
 		return;
 
 #ifdef CONFIG_DEBUG
-	qDebug("drumkv1widget::updateSchedNotify(%d, %d)", stype, sid);
+	qDebug("drumkv1widget::updateSchedNotify(%d, 0x%04x)", stype, sid);
 #endif
 
 	switch (drumkv1_sched::Type(stype)) {
+	case drumkv1_sched::MidiIn:
+		if (sid >= 0)
+			m_ui.Elements->midiInLedNote(sid & 0x7f, (sid >> 7) & 0x7f);
+		else
+		if (pDrumkUi->midiInCount() > 0) {
+			m_ui.StatusBar->midiInLed(true);
+			QTimer::singleShot(200, this, SLOT(midiInLedTimeout()));
+		}
+		break;
 	case drumkv1_sched::Controller: {
 		drumkv1widget_control *pInstance
 			= drumkv1widget_control::getInstance();
@@ -1366,6 +1437,13 @@ void drumkv1widget::updateSchedNotify ( int stype, int sid )
 }
 
 
+// MIDI In LED timeout.
+void drumkv1widget::midiInLedTimeout (void)
+{
+	m_ui.StatusBar->midiInLed(false);
+}
+
+
 // Menu actions.
 void drumkv1widget::helpConfigure (void)
 {
@@ -1373,13 +1451,7 @@ void drumkv1widget::helpConfigure (void)
 	if (pDrumkUi == NULL)
 		return;
 
-	drumkv1widget_config form(this);
-
-	// Set controllers&&programs database...
-	form.setControls(pDrumkUi->controls());
-	form.setPrograms(pDrumkUi->programs());
-
-	form.exec();
+	drumkv1widget_config(pDrumkUi, this).exec();
 }
 
 
@@ -1410,12 +1482,11 @@ void drumkv1widget::helpAbout (void)
 	sText += "<b>" DRUMKV1_TITLE "</b> - " + tr(DRUMKV1_SUBTITLE) + "<br />\n";
 	sText += "<br />\n";
 	sText += tr("Version") + ": <b>" CONFIG_BUILD_VERSION "</b><br />\n";
-	sText += "<small>" + tr("Build") + ": " CONFIG_BUILD_DATE "</small><br />\n";
-	QStringListIterator iter(list);
-	while (iter.hasNext()) {
+//	sText += "<small>" + tr("Build") + ": " CONFIG_BUILD_DATE "</small><br />\n";
+	if (!list.isEmpty()) {
 		sText += "<small><font color=\"red\">";
-		sText += iter.next();
-		sText += "</font></small><br />";
+		sText += list.join("<br />\n");
+		sText += "</font></small><br />\n";
 	}
 	sText += "<br />\n";
 	sText += tr("Website") + ": <a href=\"" DRUMKV1_WEBSITE "\">" DRUMKV1_WEBSITE "</a><br />\n";
@@ -1446,7 +1517,7 @@ void drumkv1widget::updateDirtyPreset ( bool bDirtyPreset )
 	if (pDrumkUi)
 		pDrumkUi->updatePreset(bDirtyPreset);
 
-	m_ui.StatusBar->setModified(bDirtyPreset);
+	m_ui.StatusBar->modified(bDirtyPreset);
 	m_ui.Preset->setDirtyPreset(bDirtyPreset);
 }
 
@@ -1454,9 +1525,9 @@ void drumkv1widget::updateDirtyPreset ( bool bDirtyPreset )
 // Param knob context menu.
 void drumkv1widget::paramContextMenu ( const QPoint& pos )
 {
-	drumkv1widget_knob *pKnob
-		= qobject_cast<drumkv1widget_knob *> (sender());
-	if (pKnob == NULL)
+	drumkv1widget_param *pParam
+		= qobject_cast<drumkv1widget_param *> (sender());
+	if (pParam == NULL)
 		return;
 
 	drumkv1_ui *pDrumkUi = ui_instance();
@@ -1476,9 +1547,9 @@ void drumkv1widget::paramContextMenu ( const QPoint& pos )
 		QIcon(":/images/drumkv1_control.png"),
 		tr("MIDI &Controller..."));
 
-	if (menu.exec(pKnob->mapToGlobal(pos)) == pAction) {
-		const drumkv1::ParamIndex index = m_knobParams.value(pKnob);
-		const QString& sTitle = pKnob->toolTip();
+	if (menu.exec(pParam->mapToGlobal(pos)) == pAction) {
+		const drumkv1::ParamIndex index = m_knobParams.value(pParam);
+		const QString& sTitle = pParam->toolTip();
 		drumkv1widget_control::showInstance(pControls, index, sTitle, this);
 	}
 }
