@@ -1,7 +1,7 @@
 // drumkv1widget_elements.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -27,12 +27,14 @@
 
 #include "drumkv1_sample.h"
 
-#include <QAbstractItemModel>
 #include <QHeaderView>
 #include <QFileInfo>
 #include <QMimeData>
 #include <QDrag>
 #include <QUrl>
+#include <QIcon>
+#include <QPixmap>
+#include <QTimer>
 
 #include <QDragEnterEvent>
 #include <QDragMoveEvent>
@@ -42,66 +44,46 @@
 //----------------------------------------------------------------------------
 // drumkv1widget_elements_model -- List model.
 
-class drumkv1widget_elements_model : public QAbstractItemModel
-{
-public:
-
-	// Constructor.
-	drumkv1widget_elements_model(drumkv1_ui *pDrumkUi, QObject *pParent = NULL);
-
-	// Concretizers (virtual).
-	int rowCount(const QModelIndex& parent = QModelIndex()) const;
-	int columnCount(const QModelIndex& parent = QModelIndex()) const;
-
-	QVariant headerData(int section, Qt::Orientation orient, int role) const;
-	QVariant data(const QModelIndex& index, int role) const;
-
-	QModelIndex index(int row, int column,
-		const QModelIndex& parent = QModelIndex()) const;
-
-	QModelIndex parent(const QModelIndex&) const;
-
-	void reset();
-
-	// Accessor specific.
-	drumkv1_ui *instance() const;
-
-protected:
-
-	// Other specifics
-	drumkv1_element *elementFromIndex(const QModelIndex& index) const;
-
-	QString itemDisplay(const QModelIndex& index) const;
-	QString itemToolTip(const QModelIndex& index) const;
-
-	int columnAlignment(int column) const;
-
-private:
-
-	// Model variables.
-	QStringList m_headers;
-
-	drumkv1_ui *m_pDrumkUi;
-};
-
-
 // Constructor.
 drumkv1widget_elements_model::drumkv1widget_elements_model (
 	drumkv1_ui *pDrumkUi, QObject *pParent )
 	: QAbstractItemModel(pParent), m_pDrumkUi(pDrumkUi)
 {
+	QIcon icon;
+
+	icon.addPixmap(
+		QPixmap(":/images/ledOff.png"), QIcon::Normal, QIcon::Off);
+	icon.addPixmap(
+		QPixmap(":/images/ledOn.png"), QIcon::Normal, QIcon::On);
+
+	m_pixmaps[0] = new QPixmap(
+		icon.pixmap(12, 12, QIcon::Normal, QIcon::Off));
+	m_pixmaps[1] = new QPixmap(
+		icon.pixmap(12, 12, QIcon::Normal, QIcon::On));
+
 	m_headers
 		<< tr("Element")
 		<< tr("Sample");
 
+	for (int i = 0; i < MAX_NOTES; ++i)
+		m_notes_on[i] = 0;
+
 	reset();
 }
 
 
+// Destructor.
+drumkv1widget_elements_model::~drumkv1widget_elements_model (void)
+{
+	delete m_pixmaps[1];
+	delete m_pixmaps[0];
+}
+
+
 int drumkv1widget_elements_model::rowCount (
 	const QModelIndex& /*parent*/ ) const
 {
-	return 128;
+	return MAX_NOTES;
 }
 
 
@@ -133,6 +115,10 @@ QVariant drumkv1widget_elements_model::data (
 	const QModelIndex& index, int role ) const
 {
 	switch (role) {
+	case Qt::DecorationRole:
+		if (index.column() == 0)
+			return *m_pixmaps[m_notes_on[index.row()] > 0 ? 1 : 0];
+		break;
 	case Qt::DisplayRole:
 		return itemDisplay(index);
 	case Qt::TextAlignmentRole:
@@ -172,6 +158,44 @@ drumkv1_ui *drumkv1widget_elements_model::instance (void) const
 }
 
 
+void drumkv1widget_elements_model::midiInLedNote ( int key, int vel )
+{
+	if (vel > 0) {
+		m_notes_on[key] = vel;
+		midiInLedUpdate(key);
+	}
+	else
+	if (m_notes_on[key] > 0) {
+		m_notes_off.append(key);
+		QTimer::singleShot(200, this, SLOT(midiInLedTimeout()));
+	}
+}
+
+
+void drumkv1widget_elements_model::midiInLedTimeout (void)
+{
+	QListIterator<int> iter(m_notes_off);
+	while (iter.hasNext()) {
+		const int key = iter.next();
+		midiInLedUpdate(key);
+		m_notes_on[key] = 0;
+	}
+
+	m_notes_off.clear();
+}
+
+
+void drumkv1widget_elements_model::midiInLedUpdate ( int key )
+{
+	const QModelIndex& index = drumkv1widget_elements_model::index(key, 0);
+#if QT_VERSION >= 0x050100
+	emit dataChanged(index, index, QVector<int>() << Qt::DecorationRole);
+#else
+	emit dataChanged(index, index);
+#endif
+}
+
+
 void drumkv1widget_elements_model::reset (void)
 {
 #if QT_VERSION < 0x050000
@@ -186,7 +210,6 @@ void drumkv1widget_elements_model::reset (void)
 QString drumkv1widget_elements_model::itemDisplay (
 	const QModelIndex& index ) const
 {
-	const QString sDash('-');
 	switch (index.column()) {
 	case 0: // Element.
 		return drumkv1widget::completeNoteName(index.row());
@@ -200,7 +223,7 @@ QString drumkv1widget_elements_model::itemDisplay (
 				return tr("(None)");
 		}
 	}
-	return sDash;
+	return QString('-');
 }
 
 
@@ -231,10 +254,9 @@ int drumkv1widget_elements_model::columnAlignment( int /*column*/ ) const
 
 // Constructor.
 drumkv1widget_elements::drumkv1widget_elements ( QWidget *pParent )
-	: QTreeView(pParent), m_pModel(NULL)
+	: QTreeView(pParent), m_pModel(NULL),
+		m_pDragSample(NULL), m_iDirectNoteOn(-1)
 {
-	m_pDragSample = NULL;
-
 	resetDragState();
 }
 
@@ -271,12 +293,7 @@ void drumkv1widget_elements::setInstance ( drumkv1_ui *pDrumkUi )
 	QTreeView::setAcceptDrops(true);
 
 	QHeaderView *pHeader = QTreeView::header();
-	//	pHeader->setDefaultAlignment(Qt::AlignLeft);
-#if QT_VERSION >= 0x050000
-	pHeader->setSectionResizeMode(QHeaderView::ResizeToContents);
-#else
-	pHeader->setResizeMode(QHeaderView::ResizeToContents);
-#endif
+	pHeader->setDefaultAlignment(Qt::AlignLeft);
 	pHeader->setStretchLastSection(true);
 
 	// Element selectors
@@ -325,8 +342,14 @@ void drumkv1widget_elements::doubleClicked ( const QModelIndex& index )
 void drumkv1widget_elements::mousePressEvent ( QMouseEvent *pMouseEvent )
 {
 	if (pMouseEvent->button() == Qt::LeftButton) {
-		m_dragState = DragStart;
-		m_posDrag = pMouseEvent->pos();
+		const QPoint& pos = pMouseEvent->pos();
+		if (pos.x() > 0 && pos.x() < 16) {
+			directNoteOn(QTreeView::indexAt(pos).row());
+			return; // avoid double-clicks...
+		} else {
+			m_dragState = DragStart;
+			m_posDrag = pos;
+		}
 	}
 
 	QTreeView::mousePressEvent(pMouseEvent);
@@ -363,6 +386,8 @@ void drumkv1widget_elements::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 {
 	QTreeView::mouseReleaseEvent(pMouseEvent);
 
+	directNoteOff();
+
 	m_pDragSample = NULL;
 	resetDragState();
 }
@@ -433,6 +458,8 @@ void drumkv1widget_elements::refresh (void)
 
 	m_pModel->reset();
 
+	QTreeView::header()->resizeSections(QHeaderView::ResizeToContents);
+
 	pSelectionModel->setCurrentIndex(index, QItemSelectionModel::NoUpdate);
 }
 
@@ -444,4 +471,52 @@ QSize drumkv1widget_elements::sizeHint (void) const
 }
 
 
+// MIDI input status update
+void drumkv1widget_elements::midiInLedNote ( int key, int vel )
+{
+	if (m_pModel)
+		m_pModel->midiInLedNote(key, vel);
+}
+
+
+// Direct note-on/off methods.
+void drumkv1widget_elements::directNoteOn ( int key )
+{
+	if (m_pModel == NULL || key < 0)
+		return;
+
+	drumkv1_ui *pDrumkUi = m_pModel->instance();
+	if (pDrumkUi == NULL)
+		return;
+
+	drumkv1_sample *pSample = pDrumkUi->sample();
+	if (pSample == NULL)
+		return;
+
+	const float v = pDrumkUi->paramValue(drumkv1::DEF1_VELOCITY);
+	const int vel = int(79.375f * v + 47.625f) & 0x7f;
+	pDrumkUi->directNoteOn(key, vel); // note-on!
+	m_iDirectNoteOn = key;
+
+	const float srate_ms = 0.001f * pSample->sampleRate();
+	const int timeout_ms = int(float(pSample->length() >> 1) / srate_ms);
+	QTimer::singleShot(timeout_ms, this, SLOT(directNoteOff()));
+}
+
+
+void drumkv1widget_elements::directNoteOff (void)
+{
+	if (m_pModel == NULL || m_iDirectNoteOn < 0)
+		return;
+
+	drumkv1_ui *pDrumkUi = m_pModel->instance();
+	if (pDrumkUi == NULL)
+		return;
+
+	pDrumkUi->directNoteOn(m_iDirectNoteOn, 0); // note-off!
+
+	m_iDirectNoteOn = -1;
+}
+
+
 // end of drumkv1widget_elements.cpp
