@@ -1,7 +1,7 @@
 // drumkv1widget_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -70,10 +70,10 @@ drumkv1widget_jack::drumkv1widget_jack ( drumkv1_jack *pDrumk )
 		QApplication::setStyle(QStyleFactory::create(pConfig->sCustomStyleTheme));
 
 	// Initialize (user) interface stuff...
-	m_pDrumkUi = new drumkv1_ui(m_pDrumk);
+	m_pDrumkUi = new drumkv1_ui(m_pDrumk, false);
 
 	// May initialize the scheduler/work notifier.
-	initSchedNotifier();
+	openSchedNotifier();
 
 	// Initialize preset stuff...
 	//initPreset();
